let SkylinkWrapper = {
    skylink: undefined,

    /**
     * Initializes the SkylinkWrapper. Internally, it executes
     * a command of queries to initialize the Skylink events.
     *
     * @param options Attributes to initialize Skylink framework.
     */
    init: function(options) {
        this.initWindowConfigs(options);
        this.instantiateSkylinkFramework();
        this.initOnMediaAccessSuccessEvent();
        this.initIncomingStreamEvent();
        this.initStreamEndedEvent();
        this.initPeerLeftEvent();
        this.start();
    },

    /**
     * It adds the options to the window. This is needed
     * by the Skylink framework.
     *
     * @params options Options to initialize Skylink framework.
     */
    initWindowConfigs: (options) => {
        window.config = {
            appKey: options.appKey || '',
            appKeySecret: options.appKeySecret,
            defaultRoom: options.defaultRoom || 'default_room',
            enableDataChannel: options.enableDataChannel || true,
            enableIceTrickle: options.enableDataChannel || true,
            audioFallback: options.audioFallback || true,
            forceSSL: options.forceSSL || true,
            audio: options.audio || true,
            video: options.video || true,
            localVideoEl: options.localVideoEl || 'localVideoEl',
            remoteContainerEl: options.remoteContainerEl || 'remoteContainerEl'
        };
    },

    instantiateSkylinkFramework: () => {
        skylink = new Skylink();
    },

    /**
     * It initializes the 'mediaAccessSuccess' event.
     */
    initOnMediaAccessSuccessEvent: () => {
        skylink.on('mediaAccessSuccess', (stream) => {
            attachMediaStream(document.getElementById(window.config.localVideoEl), stream);
            console.log('mediaAccessSuccess');
        });
    },

    /**
     * It initializes the 'incomingStream' event.
     */
    initIncomingStreamEvent: () => {
        skylink.on('incomingStream', (peerId, stream, isSelf, peerInfo) => {
            if (!isSelf) {
                console.log('addPeerStream');
                let DOMRemoteVideo = document.getElementById('remote_' + peerId);

                if (!DOMRemoteVideo) {
                    DOMRemoteVideo = document.createElement('video');
                    DOMRemoteVideo.setAttribute('style', 'width: 320px; height: 240px;');

                    if (window.webrtcDetectedBrowser !== 'IE')
                        DOMRemoteVideo.setAttribute('autoplay', 'autoplay');

                    DOMRemoteVideo.setAttribute('id', 'remote_' + peerId);
                    document.getElementById(window.config.remoteContainerEl).appendChild(DOMRemoteVideo);

                    DOMRemoteVideo.onclick = () => {
                        SkylinkDemo.refreshConnection(peerId);
                    };
                }

                attachMediaStream(DOMRemoteVideo, stream);
            }
        });
    },

    /**
     * It initializes the 'streamEnded' event.
     */
    initStreamEndedEvent: () => {
        skylink.on('streamEnded', (peerID, peerInfo, isSelf) => {
            if (!isSelf) {
                console.log('streamEnded');
                let DOMvideo = document.getElementById('remote_' + peerID);

                try {
                    DOMvideo.src = '';
                    document.getElementById(window.config.remoteContainerEl).removeChild(DOMvideo);
                } catch(error) {
                    console.log(error);
                }
            }
        });
    },

    /**
     * It initializes the 'peerLeft' event.
     */
    initPeerLeftEvent: () => {
        skylink.on('peerLeft', (peerID) => {
            console.log('peerLeft');
            let DOMvideo = document.getElementById('remote_' + peerID);

            try {
                DOMvideo.src = '';
                document.getElementById(window.config.remoteContainerEl).removeChild(DOMvideo);
            } catch(error) {
                console.log(error);
            }
        });
    },

    /**
     * It executes the init function of instance of Skylink framework.
     */
    start: function () {
        skylink.init(config, (error, success) => {
            if (success)
                skylink.shareScreen();

                skylink.joinRoom({
                    audio: config.audio,
                    video: config.video
                });

                setTimeout(() => {
                    try {
                        console.log("Start recording")
                        skylink.startRecording();
                    } catch(e) {
                        console.log("Error start recording:", e);
                    }
                }, 2000);


                setTimeout(() => {
                    try {
                        console.log("Stop recording")
                        skylink.stopRecording();
                    } catch(e) {
                        console.log("Error stop recording:", e);
                    }
                }, 12000);
        });
    }

}

